==> pytagsfs_0.8.0.orig.tar.gz: pytagsfs-0.8.0/modules/pytagsfs/sourcetreemon/pyinotify_.py <==
# Copyright (c) 2007-2008 Forest Bond.
# This file is part of the pytagsfs software package.
#
# pytagsfs is free software; you can redistribute it and/or modify it under the
# terms of the GNU General Public License version 2 as published by the Free
# Software Foundation.
#
# A copy of the license has been included in the COPYING file.

import os

from threading import RLock

from pytagsfs.util import nonatomic, attr_named

from pytagsfs.sourcetreemon import SourceTreeMonitor
from pytagsfs.sourcetreemon.deferred import DeferredSourceTreeMonitor
from pytagsfs.exceptions import Error, MissingDependency

from pytagsfs.debug import (
  log_debug,
  ignore_exceptions,
  exceptionally_logged,
  verbosely_logged,
)

################################################################################

class PyInotifySourceTreeMonitor(SourceTreeMonitor):
    lock = None

    notifier = None
    watch_manager = None
    event_processor = None
    watch_descriptors = None

    def __init__(self):
        try:
            import pyinotify
        except ImportError:
            raise MissingDependency('pyinotify')

        self.pyinotify = pyinotify
        self.event_mask = (
          pyinotify.EventsCodes.IN_DELETE |
          pyinotify.EventsCodes.IN_CREATE |
          pyinotify.EventsCodes.IN_MOVED_FROM |
          pyinotify.EventsCodes.IN_MOVED_TO |
          pyinotify.EventsCodes.IN_MODIFY |
          pyinotify.EventsCodes.IN_ATTRIB
        )

        self.lock = RLock()

    def start(self, debug = False):
        if debug:
            self.pyinotify.VERBOSE = True

        self._build_event_processor()

        self.watch_descriptors = { }
        self.watch_manager = self.pyinotify.WatchManager()

        self._build_notifier()

    def _build_event_processor(self):
        class PyInotifyEventProcessor(self.pyinotify.ProcessEvent):
            def __init__(self, source_tree_mon):
                super(PyInotifyEventProcessor, self).__init__()
                self.source_tree_mon = source_tree_mon

            def get_event_path(self, event):
                return os.path.join(event.path, event.name)

            @ignore_exceptions(Error)
            @exceptionally_logged
            @verbosely_logged
            def process_default(self, event):
                pass

            @ignore_exceptions(Error)
            @exceptionally_logged
            @verbosely_logged
            def _process_add(self, event):
                self.source_tree_mon.add_cb(
                  self.get_event_path(event), event.is_dir)

            @ignore_exceptions(Error)
            @exceptionally_logged
            @verbosely_logged
            def _process_remove(self, event):
                self.source_tree_mon.remove_cb(
                  self.get_event_path(event), event.is_dir)

            @ignore_exceptions(Error)
            @exceptionally_logged
            @verbosely_logged
            def _process_update(self, event):
                self.source_tree_mon.update_cb(
                  self.get_event_path(event), event.is_dir)

            process_IN_DELETE = _process_remove
            process_IN_CREATE = _process_add
            process_IN_MOVED_FROM = _process_remove
            process_IN_MOVED_TO = _process_add
            process_IN_MODIFY = _process_update
            process_IN_ATTRIB = _process_update

        self.event_processor = PyInotifyEventProcessor(self)

    def _build_notifier(self):
        self.notifier = self.pyinotify.Notifier(
          self.watch_manager, self.event_processor)

################################################################################

    # SourceTreeMonitor API

    @nonatomic(attr_named('lock'))
    def stop(self):
        self.notifier.stop()

    def add_source_dir(self, real_path):
        self.watch_dir(real_path)

    def remove_source_dir(self, real_path):
        self.unwatch_dir(real_path)

    def add_source_file(self, real_path):
        pass

    def remove_source_file(self, real_path):
        pass

    def supports_threads(self):
        return True

    def supports_writes(self):
        return True

    def can_handle_fork(self):
        return True

    def fileno(self):
        return self.notifier._fd

    @nonatomic(attr_named('lock'))
    def process_events(self):
        if self.notifier.check_events(0):
            self.notifier.read_events()
            self.notifier.process_events()

################################################################################

    @nonatomic(attr_named('lock'))
    def watch_dir(self, real_path):
        if real_path in self.watch_descriptors:
            # already watching
            return

        wds = self.watch_manager.add_watch(real_path, self.event_mask)
        wd = wds[real_path]

        if wd > 0:
            # watch successful
            self.watch_descriptors[real_path] = wd
        else:
            # watch unsuccessful, clean up
            self.watch_manager.rm_watch(wd)

    @nonatomic(attr_named('lock'))
    def unwatch_dir(self, real_path):
        try:
            wd = self.watch_descriptors[real_path]
        except KeyError:
            # not watching
            return

        status = self.watch_manager.rm_watch(wd)
        if status[wd]:
            # successfully removed watch
            del self.watch_descriptors[real_path]

################################################################################

class DeferredPyInotifySourceTreeMonitor(
  DeferredSourceTreeMonitor, PyInotifySourceTreeMonitor):
    pass

==> pytagsfs_0.9.0.orig.tar.gz: pytagsfs-0.9.0/modules/pytagsfs/sourcetreemon/pyinotify_.py <==
# Copyright (c) 2007-2008 Forest Bond.
# This file is part of the pytagsfs software package.
#
# pytagsfs is free software; you can redistribute it and/or modify it under the
# terms of the GNU General Public License version 2 as published by the Free
# Software Foundation.
#
# A copy of the license has been included in the COPYING file.

import os

from threading import RLock

from pytagsfs.util import nonatomic, attr_named

from pytagsfs.sourcetreemon import SourceTreeMonitor
from pytagsfs.sourcetreemon.deferred import DeferredSourceTreeMonitor
from pytagsfs.exceptions import (
  Error,
  MissingDependency,
  ComponentError,
)

from pytagsfs.debug import ignore_exceptions


class PyInotifySourceTreeMonitor(SourceTreeMonitor):
    lock = None

    notifier = None
    watch_manager = None
    event_processor = None
    watch_descriptors = None

    def __init__(self):
        try:
            import pyinotify
        except ImportError:
            raise MissingDependency('pyinotify')

        self.pyinotify = pyinotify
        self.event_mask = (
          pyinotify.EventsCodes.IN_DELETE |
          pyinotify.EventsCodes.IN_CREATE |
          pyinotify.EventsCodes.IN_MOVED_FROM |
          pyinotify.EventsCodes.IN_MOVED_TO |
          pyinotify.EventsCodes.IN_MODIFY |
          pyinotify.EventsCodes.IN_ATTRIB
        )

        self.lock = RLock()

    def start(self, debug = False):
        if debug:
            self.pyinotify.VERBOSE = True

        self._build_event_processor()

        self.watch_descriptors = {}

        try:
            self.watch_manager = self.pyinotify.WatchManager()
        except OSError, e:
            raise ComponentError(u'failed to initialize inotify: %s' % repr(e))

        self._build_notifier()

    def _build_event_processor(self):
        class PyInotifyEventProcessor(self.pyinotify.ProcessEvent):
            def __init__(self, source_tree_mon):
                super(PyInotifyEventProcessor, self).__init__()
                self.source_tree_mon = source_tree_mon

            def get_event_path(self, event):
                return os.path.join(event.path, event.name)

            @ignore_exceptions(Error)
            def process_default(self, event):
                pass

            @ignore_exceptions(Error)
            def _process_add(self, event):
                self.source_tree_mon.add_cb(
                  self.get_event_path(event), event.is_dir)

            @ignore_exceptions(Error)
            def _process_remove(self, event):
                self.source_tree_mon.remove_cb(
                  self.get_event_path(event), event.is_dir)

            @ignore_exceptions(Error)
            def _process_update(self, event):
                self.source_tree_mon.update_cb(
                  self.get_event_path(event), event.is_dir)

            process_IN_DELETE = _process_remove
            process_IN_CREATE = _process_add
            process_IN_MOVED_FROM = _process_remove
            process_IN_MOVED_TO = _process_add
            process_IN_MODIFY = _process_update
            process_IN_ATTRIB = _process_update

        self.event_processor = PyInotifyEventProcessor(self)

    def _build_notifier(self):
        self.notifier = self.pyinotify.Notifier(
          self.watch_manager, self.event_processor)

    @nonatomic(attr_named('lock'))
    def watch_dir(self, real_path):
        if real_path in self.watch_descriptors:
            # already watching
            return

        wds = self.watch_manager.add_watch(real_path, self.event_mask)
        wd = wds[real_path]

        if wd > 0:
            # watch successful
            self.watch_descriptors[real_path] = wd
        else:
            # watch unsuccessful, clean up
            self.watch_manager.rm_watch(wd)

    @nonatomic(attr_named('lock'))
    def unwatch_dir(self, real_path):
        try:
            wd = self.watch_descriptors[real_path]
        except KeyError:
            # not watching
            return

        status = self.watch_manager.rm_watch(wd)
        if status[wd]:
            # successfully removed watch
            del self.watch_descriptors[real_path]

    ### SourceTreeMonitor API:

    @nonatomic(attr_named('lock'))
    def stop(self):
        self.notifier.stop()

    def add_source_dir(self, real_path):
        self.watch_dir(real_path)

    def remove_source_dir(self, real_path):
        self.unwatch_dir(real_path)

    def add_source_file(self, real_path):
        pass

    def remove_source_file(self, real_path):
        pass

    def supports_threads(self):
        return True

    def supports_writes(self):
        return True

    def can_handle_fork(self):
        return True

    def fileno(self):
        return self.notifier._fd

    @nonatomic(attr_named('lock'))
    def process_events(self):
        if self.notifier.check_events(0):
            self.notifier.read_events()
            self.notifier.process_events()


class DeferredPyInotifySourceTreeMonitor(
  DeferredSourceTreeMonitor, PyInotifySourceTreeMonitor):
    pass
